mod notification;
use notification::{print_notif, to_notif};

use futures_util::stream::TryStreamExt;
use zbus::{Connection, Result};

#[tokio::main]
async fn main() -> Result<()> {
    let connection = Connection::session().await?;
    connection
        .request_name("org.freedesktop.Notifications") // Requesting dbus for this service name. Any other services/progs using this name should be stopped/disabled before this
        .await?;

    let mut stream = zbus::MessageStream::from(&connection); // Convert connection to a MessageStream, yields Message items

    while let Some(msg) = stream.try_next().await? {
        // Check if the message is a method call to the "Notify" method
        if let Some(member) = msg.header().member() {
            let member_name = member.as_str();
            match member_name {
                "GetServerInformation" => {
                    // Respond with server information
                    let response = ("SNot", "candifloss.cc", "0.1.0", "0.1.0"); // (name, vendor, version, spec_version)
                    connection.reply(&msg, &response).await?;
                    println!("Request received: {member}\n\tName: SNot\n\tVendor: candifloss.cc\n\tVersion: 0.1.0\n\tSpec_version: 0.1.0");
                }
                "GetCapabilities" => {
                    // Respond with supported capabilities
                    let capabilities = vec!["actions", "body", "body-hyperlinks"];
                    connection.reply(&msg, &capabilities).await?;
                    println!("Request received: {member}\n\tCapabilities: {capabilities:?}");
                }
                "Notify" => {
                    // Handle new received notif
                    println!("New notification:");
                    let msg_body = msg.body();
                    // get the app_name, summary, body, etc. from the msg_body
                    let notif = to_notif(&msg_body)?;
                    // Print the notif
                    print_notif(&notif);
                    println!("{}", notif.urgency());
                    // Done. Respond to the client with a notification ID
                    let notification_id: u32 = 1; // This could be incremented or generated. Do it l8r
                    connection.reply(&msg, &notification_id).await?; // The client will disconnect when it gets this response
                }
                _ => {
                    println!("Unhandled method: {member}");
                }
            }
        }
    }

    Ok(())
}
